function [ids_cores_profiles,ids_cores_profiles_description,varargout] = tcv_get_ids_core_profiles(shot,ids_equil_empty, gdat_params,varargin);
%
%  [ids_cores_profiles,ids_cores_profiles_description,varargout] = tcv_get_ids_cores_profiles(shot,ids_equil_empty,varargin);
%
%
% gdat_params: gdat_data.gdat_params to get all params passed from original call, in particular error_bar options
%
%
% error_bar: from gdat_params.error_bar
%            'delta' (default): error_bar to be added inserted in "upper" only as mentioned in description
%            'delta_with_lower' : error_bar (abs) inserted in both lower and upper
%            'added': value already added to data: upper/lower = data +/- error_bar
%

error_bar = 'delta';
if exist('gdat_params') && isfield(gdat_params,'error_bar') && ~isempty(gdat_params.error_bar)
  error_bar = gdat_params.error_bar;
end

machine = 'tcv';
tens_time = -1;
tens_rho = -0.1;

[ids_cores_profiles, params_cores_profiles] = tcv_ids_headpart(shot,ids_equil_empty,'cores_profiles',varargin{:});
ids_cores_profiles_description = [];

% base all from times fro nete_rho (which should be conf by default)
temp_1d.ne_rho = gdat(params_cores_profiles.shot,'ne_rho','machine',machine,'fit',1);
temp_1d.te_rho = gdat(params_cores_profiles.shot,'te_rho','machine',machine,'fit',1);
% compute grids_1d for fit rhopol grid, need a gdat_data like structure
temp_1d.fit.te_rho = temp_1d.te_rho.fit;
temp_1d.fit.te_rho.gdat_params = temp_1d.te_rho.gdat_params;
temp_1d.fit.te_rho.shot = temp_1d.te_rho.shot;
temp_1d.fit.te_rho = get_grids_1d(temp_1d.fit.te_rho,1,1);

temp_1d.fit.ne_rho = temp_1d.ne_rho.fit;
ids_cores_profiles.time = temp_1d.te_rho.fit.t;
ids_cores_profiles_description.time = ['.t subfield from: [''te_rho'',''subnode'',''fit.t''] thus from ' temp_1d.te_rho.fit.data_fullpath];

% make empty cell arrays for subnodes not filled in before copying default structure to all times
ids_cores_profiles.profiles_1d{1}.ion{1}.state = {};
ids_cores_profiles.profiles_1d{1}.neutral{1}.state = {};
ids_cores_profiles.profiles_1d(1:length(ids_cores_profiles.time)) = ids_cores_profiles.profiles_1d(1);


% As a general rule, for a new substructure under the main ids, construct a local structure like:
% "global_quantities" with subfields being the relevant data to get and a local structure:
% "global_quantities_desc" which contains the same subfields themselves containing the gdat string aftre shot used
%
% vacuum_toroidal_field and time, using homogeneous
%
vacuum_toroidal_field.b0=gdat(params_cores_profiles.shot,'b0','source','liuqe','machine',machine); % to get on liuqe time array
vacuum_toroidal_field_desc.b0 = '''b0'',''source'',''liuqe''';
vacuum_toroidal_field_desc.r0 = '.r0 subfield from: [''b0'',''source'',''liuqe'']';
ids_cores_profiles.vacuum_toroidal_field.r0 = vacuum_toroidal_field.b0.r0;
ids_cores_profiles.vacuum_toroidal_field.b0 = interpos(63,vacuum_toroidal_field.b0.t,vacuum_toroidal_field.b0.data,ids_cores_profiles.time,-1);
ids_cores_profiles_description.vacuum_toroidal_field = vacuum_toroidal_field_desc;

% global_quantities data into local global_quantities.* structure with correct end names and global_quantities_desc.* with description. Use temp.* and temp_desc.* structures for temporary data

global_quantities.ip = gdat(params_cores_profiles.shot,'ip','machine',machine);
global_quantities_desc.ip = 'ip';

current_non_inductive = gdat(params_cores_profiles.shot,'ohm_data','machine',machine);
global_quantities.current_non_inductive.data = current_non_inductive.ohm.ohm_data.cd_tot.data;
global_quantities.current_non_inductive.t = current_non_inductive.ohm.ohm_data.cd_tot.t;
global_quantities_desc.current_non_inductive = current_non_inductive.ohm.ohm_data.cd_tot.label;

current_bootstrap = gdat(params_cores_profiles.shot,'bs_data','machine',machine);
global_quantities.current_bootstrap.data = current_bootstrap.bs.bs_data.cd_tot.data;
global_quantities.current_bootstrap.t = current_bootstrap.bs.bs_data.cd_tot.t;
global_quantities_desc.current_bootstrap = current_bootstrap.bs.bs_data.cd_tot.label;

global_quantities.v_loop = gdat(params_cores_profiles.shot,'vloop','machine',machine);
global_quantities_desc.v_loop = 'vloop';
global_quantities.li_3 = gdat(params_cores_profiles.shot,'li','machine',machine);
global_quantities_desc.li_3 = 'li';
global_quantities.beta_tor = gdat(params_cores_profiles.shot,'beta','machine',machine);
global_quantities_desc.beta_tor = 'beta';
global_quantities.beta_tor_norm = gdat(params_cores_profiles.shot,'betan','machine',machine);
global_quantities_desc.beta_tor_norm = 'betan';
global_quantities.beta_pol = gdat(params_cores_profiles.shot,'betap','machine',machine);
global_quantities_desc.beta_pol = 'betap';
global_quantities.energy_diamagnetic = gdat(params_cores_profiles.shot,'w_mhd','machine',machine);
global_quantities_desc.energy_diamagnetic = 'w_mhd';
global_quantities.z_eff_resistive = gdat(params_cores_profiles.shot,'results.conf:z_eff','machine',machine,'fit',1);
global_quantities_desc.z_eff_resistive = 'results.conf:z_eff';

ids_cores_profiles_description.global_quantities = global_quantities_desc;

global_quantities_fieldnames = fieldnames(global_quantities);
special_fields = {''}; % fields needing non-automatic treatments
for i=1:length(global_quantities_fieldnames)
  if ~any(strcmp(global_quantities_fieldnames{i},special_fields))
    if ~isstruct(ids_cores_profiles.global_quantities.(global_quantities_fieldnames{i}))
      ids_cores_profiles.global_quantities.(global_quantities_fieldnames{i}) = ...
          interpos(global_quantities.(global_quantities_fieldnames{i}).t, ...
          global_quantities.(global_quantities_fieldnames{i}).data,ids_cores_profiles.time,tens_time);
    else
      special_fields{end+1} = global_quantities_fieldnames{i};
    end
  end
end
% special case


%
%% profiles_1d (cannot use eqdsk since not same radial mesh)
temp_1d.area = gdat(shot,'area_rho','machine',machine);
for ir=1:length(temp_1d.area.x)
  area_cpt(ir,:) = interpos(temp_1d.area.t,temp_1d.area.data(ir,:),ids_cores_profiles.time,tens_time);
end
temp_1d.q = gdat(shot,'q_rho','machine',machine);
for ir=1:length(temp_1d.q.x)
  q_cpt(ir,:) = interpos(temp_1d.q.t,temp_1d.q.data(ir,:),ids_cores_profiles.time,tens_time);
end
it_thom = iround_os(temp_1d.te_rho.t,ids_cores_profiles.time);
for it=1:length(ids_cores_profiles.time)
  ids_cores_profiles.profiles_1d{it}.grid.rho_tor_norm = temp_1d.fit.te_rho.grids_1d.rhotornorm(:,it);
  ids_cores_profiles.profiles_1d{it}.grid.rho_tor = temp_1d.fit.te_rho.grids_1d.rhotornorm(:,it) ...
      .* temp_1d.fit.te_rho.grids_1d.rhotor_edge(it);
  ids_cores_profiles.profiles_1d{it}.grid.psi = temp_1d.fit.te_rho.grids_1d.psi(:,it);
  ids_cores_profiles.profiles_1d{it}.grid.volume = temp_1d.fit.te_rho.grids_1d.rhovolnorm(:,it).^2 ...
      .* temp_1d.fit.te_rho.grids_1d.volume_edge(it);
  ids_cores_profiles.profiles_1d{it}.grid.area = interpos(temp_1d.area.x,area_cpt(:,it),temp_1d.fit.te_rho.grids_1d.rhopolnorm, ...
          tens_rho,[1 2],[0 area_cpt(end,it)]);
  ids_cores_profiles.profiles_1d{it}.time = ids_cores_profiles.time(it);
  ids_cores_profiles.profiles_1d{it}.electrons.temperature = temp_1d.fit.te_rho.data(:,it);
  ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured = temp_1d.te_rho.data(:,it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.time_measurement = temp_1d.te_rho.t(it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.rho_tor_norm = temp_1d.te_rho.grids_1d.rhotornorm(:,it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.source = {'Thomson, interpos fit'};
  ids_cores_profiles.profiles_1d{it}.electrons.density = temp_1d.fit.ne_rho.data(:,it);
  ids_cores_profiles.profiles_1d{it}.electrons.density_thermal = ids_cores_profiles.profiles_1d{it}.electrons.density;
  ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured = temp_1d.ne_rho.data(:,it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.density_fit.time_measurement = temp_1d.ne_rho.t(it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.density_fit.rho_tor_norm = temp_1d.ne_rho.grids_1d.rhotornorm(:,it_thom(it));
  ids_cores_profiles.profiles_1d{it}.electrons.density_fit.source = {'Thomson, interpos fit'};
  ids_cores_profiles.profiles_1d{it}.electrons.pressure_thermal = 1.6022e-19.*ids_cores_profiles.profiles_1d{it}.electrons.density_thermal ...
      .* ids_cores_profiles.profiles_1d{it}.electrons.temperature;
  ids_cores_profiles.profiles_1d{it}.zeff = global_quantities.z_eff_resistive.data(it) .* ...
      ones(size(ids_cores_profiles.profiles_1d{it}.electrons.density));
end

zeff_error = 0.5;
switch error_bar
 case 'delta'
  for it=1:length(ids_cores_profiles.time)
    ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_upper = temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_upper = temp_1d.ne_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.zeff_error_upper = zeff_error .* ones(size(ids_cores_profiles.profiles_1d{it}.zeff));
  end
 case 'delta_with_lower'
  for it=1:length(ids_cores_profiles.time)
    ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_upper = temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_lower = ...
        ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_upper;
    ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_upper = temp_1d.ne_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_lower = ...
        ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_upper;
    ids_cores_profiles.profiles_1d{it}.zeff_error_upper = zeff_error .* ones(size(ids_cores_profiles.profiles_1d{it}.zeff));
    ids_cores_profiles.profiles_1d{it}.zeff_error_lower = ids_cores_profiles.profiles_1d{it}.zeff_error_upper;
  end
 case 'added'
  for it=1:length(ids_cores_profiles.time)
    ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_upper = ...
        ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured ...
        + temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured_error_lower = ...
        ids_cores_profiles.profiles_1d{it}.electrons.temperature_fit.measured ...
        - temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_upper = ...
        ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured ...
        + temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured_error_lower = ...
        ids_cores_profiles.profiles_1d{it}.electrons.density_fit.measured ...
        - temp_1d.te_rho.error_bar(:,it_thom(it));
    ids_cores_profiles.profiles_1d{it}.zeff_error_upper = min(6,ids_cores_profiles.profiles_1d{it}.zeff ...
        + zeff_error .* ones(size(ids_cores_profiles.profiles_1d{it}.zeff)));
    ids_cores_profiles.profiles_1d{it}.zeff_error_upper = max(1,ids_cores_profiles.profiles_1d{it}.zeff ...
        - zeff_error .* ones(size(ids_cores_profiles.profiles_1d{it}.zeff)));
  end
 otherwise
  error(['tcv_ids_bpol_loop: error_bar option not known: ' error_bar])
end

% ion, assume only D if no CXRS (need to ask how to check if H...)
temp_1d.cxrs_rho = gdat(params_cores_profiles.shot,'cxrs','machine',machine,'fit',1);
temp_1d.ti_conf_rho = gdat(params_cores_profiles.shot,'results.conf:ti','machine',machine,'fit',1);
temp_1d.ni_conf_rho = gdat(params_cores_profiles.shot,'results.conf:ni','machine',machine,'fit',1);
data_fullpath_raw = 'Ti(C sometimes B) from cxrs system 1 to 3';
temp_1d.ti.raw = temp_1d.cxrs_rho.ti.raw;
temp_1d.ti.raw.shot = temp_1d.cxrs_rho.shot;temp_1d.ti.raw.gdat_params = temp_1d.cxrs_rho.gdat_params;
temp_1d.ti.raw.x =temp_1d.cxrs_rho.ti.raw.rho; temp_1d.ti.raw.t =temp_1d.cxrs_rho.t;
if ~isempty(temp_1d.cxrs_rho.ti.raw.data)
  data_fullpath_fit = 'Ti from fit from cxrs thus Ti(C)';
  temp_1d.ti.raw =get_grids_1d(temp_1d.ti.raw,2,1);
else
  data_fullpath_fit = 'Ti from fit in CONF node, using Te/Ti and equilibrium Wmhd';
end
temp_1d.ti.fit = temp_1d.ti_conf_rho;
temp_1d.ti.fit =get_grids_1d(temp_1d.ti.fit,1,1);
temp_1d.ni.fit = temp_1d.ni_conf_rho;
temp_1d.ni.fit =get_grids_1d(temp_1d.ni.fit,1,1);
it_ti = iround_os(temp_1d.ti.fit.t,ids_cores_profiles.time);
% assumed 1 impurity with Zp=6
Zp = 6.;
for it=1:length(ids_cores_profiles.time)
  ids_cores_profiles.profiles_1d{it}.ion{1}.element{1}.a = 2.;
  ids_cores_profiles.profiles_1d{it}.ion{1}.element{1}.z_n = 1;
  ids_cores_profiles.profiles_1d{it}.ion{1}.element{1}.atoms_n = 1;
  ids_cores_profiles.profiles_1d{it}.ion{1}.element{1}.z_ion = 1;
  ids_cores_profiles.profiles_1d{it}.ion{1}.element{1}.label = 'D+';
  ids_cores_profiles.profiles_1d{it}.ion{1}.multiple_states_flag = 0;
  ids_cores_profiles.profiles_1d{it}.ion{1}.temperature = temp_1d.ti.fit.data(:,it_ti(it));
  ids_cores_profiles.profiles_1d{it}.ion{1}.density = (temp_1d.ni.fit.data(:,it_ti(it)).*Zp-ids_cores_profiles.profiles_1d{it}.electrons.density)./(Zp-1.);
  ids_cores_profiles.profiles_1d{it}.ion{1}.density_thermal = ids_cores_profiles.profiles_1d{it}.ion{1}.density;
  ids_cores_profiles.profiles_1d{it}.ion{1}.pressure_thermal = 1.6022e-19.*ids_cores_profiles.profiles_1d{it}.ion{1}.density_thermal ...
      .* ids_cores_profiles.profiles_1d{it}.ion{1}.temperature;
  %
  ids_cores_profiles.profiles_1d{it}.t_i_average = ids_cores_profiles.profiles_1d{it}.ion{1}.temperature;
  ids_cores_profiles.profiles_1d{it}.n_i_thermal_total = ids_cores_profiles.profiles_1d{it}.ion{1}.density_thermal;
  ids_cores_profiles.profiles_1d{it}.pressure_ion_total = 1.6022e-19 .* ids_cores_profiles.profiles_1d{it}.n_i_thermal_total ...
      .* ids_cores_profiles.profiles_1d{it}.t_i_average;
  ids_cores_profiles.profiles_1d{it}.pressure_thermal = ids_cores_profiles.profiles_1d{it}.pressure_ion_total ...
      + ids_cores_profiles.profiles_1d{it}.electrons.pressure_thermal;
  %
  % C from zeff and above Ti at this stage, should take from cxrs if available but then add something for Zeff matching above
  %
  ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.a = 12.;
  ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.z_n = 6.;
  ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.atoms_n = 1.;
  ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.z_ion = 6.;
  ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.label = 'C6+';
  ids_cores_profiles.profiles_1d{it}.ion{2}.multiple_states_flag = 0;
  ids_cores_profiles.profiles_1d{it}.ion{2}.temperature = ids_cores_profiles.profiles_1d{it}.ion{1}.temperature;
  ids_cores_profiles.profiles_1d{it}.ion{2}.density = (ids_cores_profiles.profiles_1d{it}.electrons.density - ids_cores_profiles.profiles_1d{it}.ion{1}.density) ./ ids_cores_profiles.profiles_1d{it}.ion{2}.element{1}.z_ion;
  ids_cores_profiles.profiles_1d{it}.ion{2}.density_thermal = ids_cores_profiles.profiles_1d{it}.ion{2}.density;
  ids_cores_profiles.profiles_1d{it}.ion{2}.pressure_thermal = 1.6022e-19.*ids_cores_profiles.profiles_1d{it}.ion{2}.density_thermal ...
      .* ids_cores_profiles.profiles_1d{it}.ion{2}.temperature;
  % average/sums
  ids_cores_profiles.profiles_1d{it}.t_i_average = ids_cores_profiles.profiles_1d{it}.ion{1}.temperature;
  ids_cores_profiles.profiles_1d{it}.n_i_thermal_total = ids_cores_profiles.profiles_1d{it}.ion{1}.density_thermal + ...
      ids_cores_profiles.profiles_1d{it}.ion{2}.density_thermal;
  ids_cores_profiles.profiles_1d{it}.pressure_ion_total = 1.6022e-19 .* ids_cores_profiles.profiles_1d{it}.n_i_thermal_total ...
      .* ids_cores_profiles.profiles_1d{it}.t_i_average;
  ids_cores_profiles.profiles_1d{it}.pressure_thermal = ids_cores_profiles.profiles_1d{it}.pressure_ion_total ...
      + ids_cores_profiles.profiles_1d{it}.electrons.pressure_thermal;  
end
if ~isempty(temp_1d.cxrs_rho.ti.fit.data)
  it_raw = iround_os(temp_1d.ti.raw.t,ids_cores_profiles.time);
  for it=1:length(ids_cores_profiles.time)
    % ids_cores_profiles.profiles_1d{it}.ion{1}.temperature_fit = temp_1d.ti.fit(:,it_ti(it));
    ids_cores_profiles.profiles_1d{it}.ion{1}.density_fit.source = {'from Zeff and ne profile'};
    ids_cores_profiles.profiles_1d{it}.t_i_average_fit.measured = temp_1d.ti.raw.data(:,it_raw(it));
    ids_cores_profiles.profiles_1d{it}.t_i_average_fit.source = {'from CXRS on C usually'};
  end
end

temp_1d.q = gdat(shot,'q_rho','machine',machine);
for ir=1:length(temp_1d.q.x)
  q_cpt(ir,:) = interpos(temp_1d.q.t,temp_1d.q.data(ir,:),ids_cores_profiles.time,tens_time);
end
for it=1:length(ids_cores_profiles.time)
  [ids_cores_profiles.profiles_1d{it}.q] = interpos(temp_1d.q.x,q_cpt(:,it),temp_1d.fit.te_rho.grids_1d.rhopolnorm, ...
          tens_rho,[1 0],[0 0]);
  [qfit,dqdrhotor] = interpos(ids_cores_profiles.profiles_1d{it}.grid.rho_tor,ids_cores_profiles.profiles_1d{it}.q,tens_rho,[1 0],[0 0]);
  ids_cores_profiles.profiles_1d{it}.shear = ids_cores_profiles.profiles_1d{it}.grid.rho_tor./ids_cores_profiles.profiles_1d{it}.q ...
      .* dqdrhotor;
end

if nargin <= 2
  ids_cores_profiles.code.name = ['tcv_get_ids_core_profiles, within gdat, with shot= ' num2str(shot) ];
else
  ids_cores_profiles.code.name = ['tcv_get_ids_core_profiles, within gdat, with shot= ' num2str(shot) '; varargin: ' varargin{:}];
end
ids_cores_profiles.code.output_flag = zeros(size(ids_cores_profiles.time));

% make arrays not filled in empty:
for it=1:length(ids_cores_profiles.time)
  ids_cores_profiles.profiles_1d{it}.neutral = {};
end
